@@ -1,3 +1,4 @@
+
 package MP3::Tag;
 
 # Copyright (c) 2000-2004 Thomas Geffert.  All rights reserved.
@@ -36,10 +37,19 @@ use MP3::Tag::CDDB_File;
 use MP3::Tag::ParseData;
 use MP3::Tag::LastResort;
 
-use vars qw/$VERSION %config/;
+use vars qw/$VERSION @ISA/;
+$VERSION="0.9707";
+@ISA = qw( MP3::Tag::User MP3::Tag::Site MP3::Tag::Vendor
+	   MP3::Tag::Implemenation ); # Make overridable
+*config = \%MP3::Tag::Implemenation::config;
+
+package MP3::Tag::Implemenation;
+use vars qw/%config/;
 %config = ( autoinfo	=> [qw(ParseData ID3v2 ID3v1 CDDB_File Inf filename LastResort)],
 	    cddb_files	=> [qw(audio.cddb cddb.out cddb.in)],
 	    v2title	=> [qw(TIT1 TIT2 TIT3)],
+	    composer	=> ['TCOM|a'],
+	    performer	=> ['TXXX[TPE1]|TPE1|a'],
 	    extension	=> ['\.(?!\d+\b)\w{1,4}$'],
 	    parse_data	=> [],
 	    parse_split	=> ["\n"],
@@ -53,8 +63,8 @@ use vars qw/$VERSION %config/;
 	    id3v2_sizemult	=> [512],
 	    id3v2_shrink	=> [0],
 	    id3v2_mergepadding  => [0],
-	    id3v23_unsync_size_r => [0],
 	    id3v23_unsync_size_w => [0],
+	    id3v23_unsync => [1],
 	    parse_minmatch => [0],
 	    update_length => [1],
 	    person_frames => [qw{ TEXT TCOM TXXX[TPE1] TPE1 TPE3 TOPE TOLY
@@ -78,8 +88,6 @@ use vars qw/$VERSION %config/;
   $config{encode_encoding_files} = [$e{eF}] if $e{eF};
 }
 
-$VERSION="0.9706";
-
 =pod
 
 =head1 NAME
@@ -483,6 +491,42 @@ If an optional argument C<'from'> is given, returns an array reference with
 the first element being the value, the second the tag (ID3v2 or ID3v1 or
 filename) from which the value is taken.
 
+=item composer()
+
+  $composer = $mp3->composer();		# empty string unless found
+
+composer() returns the composer.  By default, it gets from ID3v2 tag,
+otherwise returns artist.
+
+You can change the inspected fields with the config() command.
+Subject to normalization via C<translate_composer> or
+C<translate_person> configuration variables.
+
+=item performer()
+
+  $performer = $mp3->performer();		# empty string unless found
+
+performer() returns the main performer.  By default, it gets from ID3v2
+tag C<TXXX[TPE1]>, otherwise from ID3v2 tag C<TPE1>, otherwise
+returns artist.
+
+You can change the inspected fields with the config() command.
+Subject to normalization via C<translate_performer> or
+C<translate_person> configuration variables.
+
+=cut
+
+for my $elt ( qw( composer performer ) ) {
+  no strict 'refs';
+  *$elt = sub (;$) {
+    my $self = shift;
+    my $translate = ($self->get_config("translate_$elt")
+		     || $self->get_config("translate_person")
+		     || [])->[0] || sub {$_[1]};
+    my $fields = ($self->get_config($elt))->[0];
+    return &$translate($self, $self->interpolate("%{$fields}"));
+  }
+}
 
 =item config
 
@@ -537,6 +581,16 @@ will be used instead.
 regular expression to match the file extension (including the dot).  The
 default is to match 1..4 letter extensions which are not numbers.
 
+=item  composer
+
+string to put into C<%{}> to interpolate to get the composer.  Default
+is C<'TCOM|a'>.
+
+=item performer
+
+string to put into C<%{}> to interpolate to get the main performer.
+Default is C<'TXXX[TPE1]|TPE1|a'>.
+
 =item parse_data
 
 the data used by L<MP3::Tag::ParseData> handler; each option is an array
@@ -670,6 +724,12 @@ Setting these values will assume another interpretation (as in v2.4) for
 write; experimental - to test why ITunes refuse to
 handle unsyncronized tags.
 
+=item id3v23_unsync
+
+Some broken MP3 players (e.g., ITunes, at least up to v6) refuse to
+handle unsyncronized (e.g., written as the standard requires it) tags;
+they may need this to be set to FALSE.  Default: TRUE.
+
 =item encode_encoding_v1
 
 =item decode_encoding_v1
@@ -715,12 +775,13 @@ sub config {
     my $config = ref $self ? ($self->{config} ||= {%config}) : \%config;
     my @known = qw(autoinfo title artist album year comment track genre
 		   v2title cddb_files force_interpolate parse_data parse_split
+		   composer performer
 		   parse_join parse_filename_ignore_case
 		   parse_filename_merge_dots year_is_timestamp
 		   comment_remove_date extension id3v2_missing_fatal
 		   id3v2_frame_empty_ok id3v2_minpadding id3v2_sizemult
-		   id3v2_shrink id3v2_mergepadding person_frames
-		   parse_minmatch id3v23_unsync_size_w short_person
+		   id3v2_shrink id3v2_mergepadding person_frames short_person
+		   parse_minmatch id3v23_unsync id3v23_unsync_size_w
 		   encode_encoding_files encode_encoding_v1
 		   decode_encoding_v1 decode_encoding_v2
 		   decode_encoding_filename decode_encoding_files
@@ -728,6 +789,7 @@ sub config {
     my @tr = map "translate_$_", qw( title track artist album comment
 				     year genre comment_collection
 				     comment_track title_track
+				     composer performer
 				     artist_collection person );
     $conf_rex = '^(' . join('|', @known, @tr) . ')$' unless $conf_rex;
 
@@ -816,7 +878,8 @@ sub set_user ($$$) {
 
 When called with only $name as the argument, removes the specified
 frame (if it existed).  Otherwise sets the frame passing the specified
-@values to the add_frame() function of MP3::Tag::ID3v2.
+@values to the add_frame() function of MP3::Tag::ID3v2.  (The old value is
+removed.)
 
 =cut
 
@@ -849,6 +912,25 @@ sub get_id3v2_frames ($$;$) {
     $self->{ID3v2}->get_frames(@_);
 }
 
+=item delete_tag
+
+  $deleted = $mp3->delete_tag($tag);
+
+$tag should be either C<ID3v1> or C<ID3v2>.  Deletes the tag if it is present.
+Returns FALSE if the tag is not present.
+
+=cut
+
+sub delete_tag ($$) {
+    my ($self, $tag) = (shift, shift);
+    $self->get_tags;
+    die "Unexpected tag type '$tag'" unless $tag =~ /^ID3v[12]$/;
+    return unless exists $self->{$tag};
+    my $res = $self->{$tag}->remove_tag();
+    $res = ($res >= 0) if $tag eq 'ID3v1'; # -1 on error
+    $res or die "Error deleting tag `$tag'";
+}
+
 =item is_id3v2_modified
 
   $frame = $mp3->is_id3v2_modified();
@@ -873,6 +955,9 @@ Descriptors, list of wanted Languages, and possibly the new contents - with
 C<undef> meaning deletion).  For read-only access it returns C<undef> if no
 ID3v2 tag is present.
 
+If new context is specified, all the existing frames matching the specification
+are deleted.
+
 =item have_id3v2_frame
 
   $have_it = $mp3->have_id3v2_frame($fname, $descrs, $langs);
@@ -1094,8 +1179,8 @@ The one-letter ESCAPEs are replaced by
 		u	frames
 
 
-Additionally, ESCAPE can be a string (with all backslashes and curlies escaped)
-enclosed in curly braces C<{}>.  The interpretation is the following:
+Additionally, ESCAPE can be a string enclosed in curly braces C<{}>.
+The interpretation is the following:
 
 =over 4
 
@@ -1158,8 +1243,7 @@ array.
 =item *
 
 If string starts with C<FNAME:>: if frame FNAME does not exists, the escape
-is ignored; otherwise the rest of the string is reinterpreted (after stripping
-backslashes from backslashes and curlies).
+is ignored; otherwise the rest of the string is reinterpreted.
 
 =item *
 
@@ -1169,8 +1253,7 @@ String starting with C<!FNAME:> are treated similarly with inverted test.
 
 If string starts with C<FNAME||>: if frame FNAME exists, the part
 after C<||> is ignored; otherwise the part before C<||> is ignored,
-and the rest is reinterpreted (after stripping backslashes from
-backslashes and curlies).
+and the rest is reinterpreted.
 
 =item *
 
@@ -1193,8 +1276,18 @@ Likewise for string starting with I<LETTER>C<|> or I<LETTER>C<||>.
 For strings of the form C<nmP[VALUE]> or C<shP[VALUE]>, I<VALUE> is
 interpolated, then normalized or shortened as a personal name
 (according to C<translate_person> or C<short_person> configuration
-setting).  Braces (C<[]>) and backslash in I<VALUE> should be
-protected by backslashes.
+setting).
+
+=item *
+
+C<composer> or C<performer> is replaced by the result of calling the
+corresponding method.
+
+=item *
+
+For strings of the form C<I(FLAGS)VALUE>, I<VALUE> is interpolated
+with flags in I<FLAGS> (see L<"interpolate_with_flags">).  If FLAGS
+does not contain C<i>, VALUE should have C<{}> and C<\> backwacked.
 
 =back
 
@@ -1223,10 +1316,24 @@ include both.)
 
   Composer: %{TCOM|a}
 
-will use the ID3v2 field C<TCOM> if present, otherwise uses C<%a>.
+will use the ID3v2 field C<TCOM> if present, otherwise uses C<%a> (this is similar to
 
-Interpolation of ID3v2 frames uses the same translation rules as
-select_id3v2_frame_by_descr().
+  Composer: %{composer}
+
+but the latter is subject to normalization, and/or configuration variables).
+
+Interpolation of ID3v2 frames uses the minimal possible non-ambiguous
+backslashing rules: the only backslashes needed are to protect the
+innermost closing delimiter (C<]> or C<}>) appearing as a literal
+character, or to protect backslashes I<immediately> preceeding such
+literal, or the closing delimiter.  E.g., the pattern equal to
+
+  %{COMM(eng)[a\b\\c\}\]end\\\]\\\\]: comment `a\b\\c\\\}]end\]\\' present}
+
+checks for the presence of comment with the descriptor C<a\b\\c\}]end\]\\>.
+Note that if you want to write this string as a Perl literal, a lot of
+extra backslashes may be needed (unless you use C<E<lt>E<lt>'FOO'>
+HERE-document).
 
 =cut
 
@@ -1284,101 +1391,173 @@ my %trans = qw(	t	title
 #	%O      Original material flag (string)
 #	%G      Musical genre (integer)
 
-my $frame_id =			# FRAM | FRAM03 | FRAM(lang)[descr]
-  qr{\w{4}(?:(?:\d\d)|(?:\([^)]*\))?(?:\[(?:\\.|[^]\\])*\])?)}s;
+my $frame_bra =			# FRAM | FRAM03 | FRAM(lang)[
+  qr{\w{4}(?:(?:\d\d)|(?:\([^)]*\))?(?:(\[)|(?=[\}:|&])))}s; # 1 group for begin-descr
+# used with offset by 1: 2: fill, 3: same, 4: $left, 5..6 width, 5: key
+my $pat_rx = qr/^%(?:(?:\((.)\)|([^-.1-9%a-zA-Z]))?(-)?(\d+))?(?:\.(\d+))?([talygcnfFeEABDNvLrqQSmsCpouM{%])/s;
 
-sub interpolate {
-    my ($self, $pattern) = @_;
+
+# $upto TRUE: parse the part including $upto char
+# Very restricted backslashitis: only $upto and \ before $upto-or-end
+# $upto defined but FALSE: interpolate only one %-escape.
+# Anyway: $_[1] is modified to remove interplated part.
+sub _interpolate ($$;$$) {
+    # goto &interpolate_flags if @_ == 3;
+    my ($self, undef, $upto, $skip) = @_; # pattern is modified, so is $_[1]
     $self->get_tags();
     my $res = "";
     my $ids;
+    die "upto=`$upto' not supported" if $upto and $upto ne ']' and $upto ne'}';
+    die "upto=`$upto' not supported with skip"
+      if $upto and not defined $upto and $skip;
+    my $cnt = ($upto or not defined $upto) ? -1 : 1; # upto eq '': 1 escape
 
-    while ($pattern =~ s/^([^%]+)|^%(?:(?:\((.)\)|([^-.1-9%a-zA-Z]))?(-)?(\d+))?(?:\.(\d+))?([talygcnfFeEABDNvLrqQSmsCpou{%])//s) {
-	$res .= $1, next if defined $1;
+    while ($cnt-- and ($upto	# undef and '' use the same code
+		       ? ($upto eq ']'
+			  ? $_[1] =~ s/^((?:[^%\\\]]|(?:\\\\)*\\\]|\\+[^\\\]]|\\\\)+)|$pat_rx//so
+			  : $_[1] =~ s/^((?:[^%\\\}]|(?:\\\\)*\\\}|\\+[^\\\}]|\\\\)+)|$pat_rx//so)
+		       : $_[1] =~ s/^([^%]+)|$pat_rx//so)) {
+        if (defined $1) {
+	  my $str = $1;
+	  if ($upto and $upto eq ']') {
+	    $str =~ s<((?:\\\\)*)(?:\\(?=\])|(?!.))>< '\\' x (length($1)/2) >ges;
+	  } elsif ($upto and $upto eq '}') {
+	    $str =~ s<((?:\\\\)*)(?:\\(?=\})|(?!.))>< '\\' x (length($1)/2) >ges;
+	  }
+	  $res .= $str, next;
+	}
 	my ($fill, $left, $minwidth, $maxwidth, $what)
 	    = ((defined $2 ? $2 : $3), $4, $5, $6, $7);
+	next if $skip and $what ne '{';
 	my $str;
-	if ($what eq '{' and $pattern =~ s/^([dD])(\d+)}//) {	# Directory
+	if ($what eq '{' and $_[1] =~ s/^([dD])(\d+)}//) {	# Directory
+	    next if $skip;
 	    if ($1 eq 'd') {
 		$str = $self->dir_component($2);
 	    } else {
 		$str = $self->dirname($2);
 	    }
-	} elsif ($what eq '{' and $pattern =~ s/^U(\d+)}//) {	# User data
+	} elsif ($what eq '{' and $_[1] =~ s/^U(\d+)}//) {	# User data
+	    next if $skip;
 	    $str = $self->get_user($1);
-	} elsif ($what eq '{' and $pattern =~ s/^(aC|tT|c[TC])}//) { # CDDB
+	} elsif ($what eq '{' and $_[1] =~ s/^(aC|tT|c[TC])}//) { # CDDB
+	    next if $skip;
 	    my $meth = $trans{$1};
 	    $str = $self->$meth();
-	} elsif ($what eq '{' and $pattern =~ s/^(!)?([talygcnfFeEABD])(:|\|\|?)((?:[^\\{}]|\\[\\{}])*)}//) {
-	    # Alternation with simple stuff
-	    my ($neg, $alt) = ($1, ($3 ne ':') && $3);
-	    die "Negation and alternation incompatible in interpolation"
-	      if $alt and $neg;
-	    $str = $self->interpolate("%$2");
-	    my $have = length($str);
-	    next if not $alt and $1 ? $have : !$have;
-	    unless ($have and $alt) {
-	      $str = $4;
-	      if ($alt and $alt ne '||') {
-		$str = $self->interpolate(length $str > 1 ? "%{$str}" : "%$str")
-	      } else {
-		$str =~ s/\\([\\{}])/$1/g;
-		$str = $self->interpolate($str);
+	} elsif ($what eq '{' and # $frame_bra has 1 group, No. 5
+		 $_[1] =~ s/^(!)?(([talygcnfFeEABD])(:|\|\|?)|$frame_bra)//) {
+	    # Alternation with simple/complicated stuff
+	    my ($id, $simple, $neg, $delim) = ($2, $3, $1, $4);
+	    if ($delim) {
+	      $id = $simple;
+	    } else {		# Frame: maybe trailed by :, |, ||, maybe not
+	      $id .= ($self->_interpolate($_[1], ']', $skip) . ']') if $5;
+	      $_[1] =~ s/^(:|\|\|?)// and $delim = $1;
+	      unless ($delim) {
+		die "Can't parse negated conditional: I see `$_[1]'" if $neg;
+		my $nonesuch = 0;
+		unless ($self->{ID3v2} or $neg) {
+		  die "No ID3v2 present"
+		    if $self->get_config('id3v2_missing_fatal');
+		  $nonesuch = 1;
+		}
+		if ($_[1] =~ s/^}//) { # frame with optional (lang)/[descr]
+		  next if $skip or $nonesuch;
+		  $str = $self->select_id3v2_frame_by_descr($id);
+		  $str = $str->{_Data} if $str and ref $str and exists $str->{_Data};
+		} elsif ($_[1] =~ /^&/o) {
+		  # join of frames with optional (language)/[descriptor]
+		  my @id = $id;
+		  while ($_[1] =~ s/^&($frame_bra)//o) {
+		    $id = $1;
+		    $id .= ($self->_interpolate($_[1], ']', $skip) . ']') if $2;
+		    next if $skip or $nonesuch;
+		    push @id, $id;
+		  }
+		  die "Can't parse &-list; I see `$_[1]'" unless $_[1] =~ s/^}//;
+		  next if $skip or $nonesuch;
+		  my @out;
+		  for my $in (@id) {
+		    $in = $self->select_id3v2_frame_by_descr($in);
+		    $in = $in->{_Data} if $in and ref $in and exists $in->{_Data};
+		    push @out, $in if defined $in and length $in;
+		  }
+		  $str = join '; ', @out;
+		} else {
+		  die "unknown frame terminator; I see `$_[1]'";
+		}
 	      }
 	    }
-	} elsif ($what eq '{' and $pattern =~ s/^ID3v1}//) {
+	    if ($delim) {
+	      # $self->_interpolate($_[1], $upto, $skip), next if $skip;
+	      my $alt = ($delim ne ':') && $delim; # FALSE or $delim
+	      die "Negation and alternation incompatible in interpolation"
+		if $alt and $neg;
+	      my $have;
+	      if ($simple) {
+		$str = $self->interpolate("%$simple");
+		$have = length($str);
+	      } else {
+		$have = $self->have_id3v2_frame_by_descr($id);
+	      }
+	      my $skipping = $skip || (not $alt and $1 ? $have : !$have);
+	      my $s;
+	      if ($alt and $alt ne '||') { # Need to prepend %
+		if ($_[1] =~ s/^([^\\])}//) { # One-char escape
+		  $s = $self->interpolate("%$1") unless $skipping;
+		} else {	# Understood with {}; prepend %{
+		  $_[1] =~ s/^/%\{/ or die;
+		  $s = $self->_interpolate($_[1], '', $skipping);
+		}
+	      } else {
+		$s = $self->_interpolate($_[1], '}', $skipping);
+	      }
+	      next if $skipping;
+	      $str = $self->select_id3v2_frame_by_descr($id)
+		if $alt and $have and not $simple;
+	      $str = $s unless $have and $alt;
+	      $str = $str->{_Data}
+		if $str and ref $str and exists $str->{_Data};
+	    }
+	} elsif ($what eq '{' and $_[1] =~ s/^ID3v1}//) {
+	    next if $skip;
 	    $str = $self->{ID3v1}->as_bin if $self->{ID3v1};
 	} elsif ($what eq '{'
-		 and $pattern =~ s/^(sh|nm)P\[((?:\\.|[^{}\\])*)\]}//s) {
-	    # Shorten personal name
-	    my $meth = ($1 eq 'sh' ? 'shorten_person' : 'normalize_person');
-	    ($str = $2) =~ s/\\([\\{}])/$1/g;
-	    $str = $self->$meth($self->interpolate($str));
-	} elsif ($what eq '{') {	# id3v2 stuff
-	    $pattern =~ s/^((?:[^\\{}]|\\[\\{}])*)}// or die "Mismatched {} in pattern `$pattern'";
+		 and $_[1] =~ s/^(sh|nm)P\[//s) {
+	    # (Short) personal name
 	    $what = $1;
-	    unless ($self->{ID3v2} or $what =~ /^!/) {
-		die "No ID3v2 present" if $self->get_config('id3v2_missing_fatal');
+	    $str = $self->_interpolate($_[1], ']', $skip);
+	    $_[1] =~ s/^\}// or die "Can't find end of ${what}P escape; I see `$_[1]'";
+	    next if $skip;
+	    my $meth = ($what eq 'sh' ? 'shorten_person' : 'normalize_person');
+	    $str = $self->$meth($str);
+	} elsif ($what eq '{' and $_[1] =~ s/^I\((\w+)\)//s) {
+	    # Interpolate
+	    my $flags = $1;
+	    if ($flags =~ s/i//) {
+	      $str = $self->_interpolate($_[1], '}', $skip);
+	    } else {
+	      $_[1] =~ s/^((?:[^\\\}]|(?:\\\\)*\\\}|\\+[^\\\}]|\\\\)*)\}//s
+	      #		$_[1] =~ s/^((?:\\.|[^{}\\])*)}//
+		or die "Can't find non-interpolated argument in `$_[1]'";
+	      next if $skip;
+	      # ($str = $1) =~ s/\\([\\{}])/$1/g;
+	      ($str = $1) =~ s<((?:\\\\)*)(?:\\(?=\})|(?!.))>< '\\' x (length($1)/2) >ges;
+	    }
+	    next if $skip;
+	    ($str) = $self->interpolate_with_flags($str, $flags);
+	} elsif ($what eq '{') {	# id3v2 wholesale, composer/performer
+	    unless ($self->{ID3v2} or $_[1] =~ /^!/) {
+		die "No ID3v2 present"
+		  if $self->get_config('id3v2_missing_fatal');
 		next;
 	    }
-	    if ($what eq 'ID3v2') { # Whole tag
-		$str = $self->{ID3v2}->as_bin if $self->{ID3v2};
-	    } elsif ($what =~ /^$frame_id$/o) {
-	        # frame with optional (language)/[descriptor]
-		$str = $self->select_id3v2_frame_by_descr($what);
-		$str = $str->{_Data} if $str and ref $str and exists $str->{_Data};
-	    } elsif ($what =~ /^(!)?($frame_id)(:|\|\|?)(.*)$/so) {#: or| or ||
-	        # alternation with frame with optional (language)/[descriptor]
-	        my ($neg, $id, $alt) = ($1, $2, ($3 ne ':') && $3);
-		die "Negation and alternation incompatible in interpolation"
-		    if $alt and $neg;
-		my $have = $self->have_id3v2_frame_by_descr($id);
-		next if not $alt and $1 ? $have : !$have;
-		$str = $4;
-		if ($alt and $have) {
-		  $str = $self->select_id3v2_frame_by_descr($id);
-		} elsif ($alt and $alt ne '||') { # |
-		  $str = $self->interpolate(length $str > 1 ? "%{$str}" : "%$str");
-		  $str = $str->{_Data}
-		    if $str and ref $str and exists $str->{_Data};
-		} else {
-		  $str =~ s/\\([\\{}])/$1/g; # Un-backwack \{ \} \\
-		  $str = $self->interpolate($str);
-		}
-	    } elsif ($what =~ /^($frame_id)&(.*)$/o) {
-	        # join of frames with optional (language)/[descriptor]
-	        my @id;
-		while (defined $what) {
-		  $what =~ /^($frame_id)(?:&(.*)|$)$/o
-		    or die "Can't parse &-list {$what}";
-		  $what = $2;
-		  my $in = $self->select_id3v2_frame_by_descr($1);
-		  $in = $in->{_Data} if $in and ref $in and exists $in->{_Data};
-		  push @id, $in if defined $in and length $in;
-		}
-		$str = join '; ', @id;
+	    if ($_[1] =~ s/ID3v2}//) { # Whole tag
+		$str = $self->{ID3v2}->as_bin if not $skip and $self->{ID3v2};
+	    } elsif ($_[1] =~ s/^(composer|performer)}//) {
+	      $str = $self->$1() unless $skip;
 	    } else {
-		die "unknown escape `$what'";
+	      die "unknown escape; I see `$_[1]'";
 	    }
 	} elsif ($what eq '%') {
 	    $str = '%';
@@ -1398,10 +1577,89 @@ sub interpolate {
 	}
 	$res .= $str;
     }
-    die "Can't parse `$pattern' during interpolation" if length $pattern;
+    if (defined $upto) {
+      not $upto or
+	($upto eq ']' ? $_[1] =~ s/^\]// : $_[1] =~ s/^\}//)
+	  or die "Can't find final delimiter `$upto': I see `$_[1]'";
+    } else {
+      die "Can't parse `$_[1]' during interpolation" if length $_[1];
+    }
     return $res;
 }
 
+sub interpolate ($$) {
+  my ($self, $pattern) = @_;	# local copy; $pattern is modified
+  $self->_interpolate($pattern);
+}
+
+
+=item interpolate_with_flags
+
+  @results = $mp3->interpolate_with_flags($pattern, $flags);
+
+Processes $pattern according to directives in the string $flags; $flags is
+split into separate flag characters; the meanings (and order of application) of
+flags are
+
+   i			interpolate via $mp3->interpolate
+   f			interpret (the result) as filename, read from file
+   F			if file does not exist, it is not an error
+   B			read is performed in binary mode (otherwise per
+				'decode_encoding_files' configuration variable)
+   l			split result per 'parse_split' configuration variable
+   n			as l, using the track-number-th element (1-based)
+				in the result
+   I			interpolate (again) via $mp3->interpolate
+   b			unless present, remove leading and trailing whitespace
+
+With C<l>, may produce multiple results.
+
+=cut
+
+sub interpolate_with_flags ($$$) {
+    my ($self, $data, $flags) = @_;
+
+    $data = $self->interpolate($data) if $flags =~ /i/;
+    if ($flags =~ /f/) {
+	local *F;
+	my $e;
+	unless (open F, "< $data") {
+	  return if $flags =~ /F/;
+	  die "Can't open file `$data' for parsing: $!";
+	}
+	if ($flags =~ /B/) {
+	  binmode F;
+	} else {
+	  my $e;
+	  if ($e = $self->get_config('decode_encoding_files') and $e->[0]) {
+	    eval "binmode F, ':encoding($e->[0])'"; # old binmode won't compile...
+	  }
+	}
+
+	local $/;
+	my $d = <F>;
+	CORE::close F or die "Can't close file `$data' for parsing: $!";
+	$data = $d;
+    }
+    my @data = $data;
+    if ($flags =~ /[ln]/) {
+	my $p = $self->get_config('parse_split')->[0];
+	@data = split $p, $data, -1;
+    }
+    if ($flags =~ /n/) {
+	my $track = $self->{parent}->track or return;
+	@data = $data[$track - 1];
+    }
+    for my $d (@data) {
+	$d = $self->{parent}->interpolate($d) if $flags =~ /I/;
+	unless ($flags =~ /b/) {
+	    $d =~ s/^\s+//;
+	    $d =~ s/\s+$//;
+	}
+    }
+    @data;
+}
+
 =item parse_rex($pattern, $string)
 
 Parse $string according to the regular expression $pattern with
@@ -1433,11 +1691,12 @@ C<'%{FRAM(lang,list)[description]}'>.
 			  $mp3->filename_nodir ) or die;
   $author = $res->{author};
 
-2-digit numbers are allowed for the track number (the leading 0 is
-stripped); 4-digit years in the range 1000..2999 are allowed for year.
-Alternatively, if option year_is_timestamp is TRUE (default), year may
-be a range of timestamps in the format understood by ID3v2 method
-year() (see L<MP3::Tag::ID3v2/"year">).
+2-digit numbers, or I<number1/number2> with number1,2 up to 999 are
+allowed for the track number (the leading 0 is stripped); 4-digit
+years in the range 1000..2999 are allowed for year.  Alternatively, if
+option year_is_timestamp is TRUE (default), year may be a range of
+timestamps in the format understood by ID3v2 method year() (see
+L<MP3::Tag::ID3v2/"year">).
 
 Currently the regular expressions with capturing parens are not supported.
 
@@ -1489,8 +1748,8 @@ sub _parse_rex_anything ($$) {
 sub _parse_rex_microinterpolate {	# $self->idem($code, $groups, $ecount)
     my ($self, $code, $groups) = (shift, shift, shift);
     return '%' if $code eq '%';
-    # In these two, allow setting to '' too...
-    push(@$groups, $code), return '((?<!\d)\d{1,2}(?!\d)|\A\Z)' if $code eq 'n';
+    # In these two, allow setting to '', and to 123/789 too...
+    push(@$groups, $code), return '((?<!\d)\d{1,2}(?:\d?/\d{1,3})?(?!\d)|\A\Z)' if $code eq 'n';
     (push @$groups, $code), return '((?<!\d)[12]\d{3}(?:(?:--|[-:/T\0,])\d(?:|\d|\d\d\d))*(?!\d)|\A\Z)'
 	if $code eq 'y' and ($self->get_config('year_is_timestamp'))->[0];
     (push @$groups, $code), return '((?<!\d)[12]\d{3}(?!\d)|\A\Z)'
@@ -1506,7 +1765,7 @@ sub _parse_rex_microinterpolate {	# $self->idem($code, $groups, $ecount)
     $_[0]++, return '(?<!\d)' . quotemeta($self->year) . '(?!\d)'
 	if $code eq '=y';
     (push @$groups, $1), return $self->_parse_rex_anything()
-	if $code =~ /^{(U\d+|\w{4}(\d\d+|(?:\([^\)]*\))?(?:\[(\\.|[^]\\])*\])?)?)}$/s;
+	if $code =~ /^{(U\d+|\w{4}(\d\d+|(?:\([^\)]*\))?(?:\[.*\])?)?)}$/s;
     # What remains is extension
     my $e = $self->get_config('extension')->[0];
     (push @$groups, $code), return "($e)" if $code eq 'E';
@@ -1520,12 +1779,28 @@ sub _parse_rex_microinterpolate {	# $self->idem($code, $groups, $ecount)
 
 sub parse_rex_prepare {
     my ($self, $pattern) = @_;
-    my ($codes, $exact) = ([], 0);
+    my ($codes, $exact, $p) = ([], 0, '');
     my $o = $pattern;
-    $pattern =~ s<%(=?{(?:[^\\{}]|\\[\\{}])*}|{U\d+}|=?.)> # (=? is correct!
-		 ( $self->_parse_rex_microinterpolate($1, $codes, $exact) )seg;
+    # (=? is correct! Group 4 is inside $frame_bra
+    while ($pattern =~ s<^([^%]+)|%(=?{(?:($frame_bra)|[^}]+})|=?.)><>so) {
+      if (defined $1) {
+	$p .= $1;
+      } else {
+	my $group = $2;
+	# description begins
+	$group .= ($self->_interpolate($pattern, ']') . ']') if $4;
+	if ($3) {
+	  $pattern =~ s/^}// or die "Can't find end of frame name, I see `$p'";
+	  $group .= '}';
+	}
+	$p .= $self->_parse_rex_microinterpolate($group, $codes, $exact);
+      }
+    }
+    die "Can't parse pattern, I see `$pattern'" if length $pattern;
+    #$pattern =~ s<%(=?{(?:[^\\{}]|\\[\\{}])*}|{U\d+}|=?.)> # (=? is correct!
+    #		 ( $self->_parse_rex_microinterpolate($1, $codes, $exact) )seg;
     my @tags = map { length == 1 ? $trans{$_} : $_ } @$codes;
-    return [$o, $pattern, \@tags, $exact];
+    return [$o, $p, \@tags, $exact];
 }
 
 sub parse_rex_match {	# pattern = [Original, Interpolated, Fields, NumExact]
@@ -1928,6 +2203,22 @@ sub DESTROY {
     }
 }
 
+my @parents = qw(User Site Vendor);
+
+@MP3::Tag::User::ISA = qw( MP3::Tag::Site MP3::Tag::Vendor
+			   MP3::Tag::Implemenation ); # Make overridable
+@MP3::Tag::Site::ISA = qw( MP3::Tag::Vendor MP3::Tag::Implemenation );
+@MP3::Tag::Vendor::ISA = qw( MP3::Tag::Implemenation );
+
+sub load_parents {
+  my $par;
+  while ($par = shift @parents) {
+    return 1 if eval "require MP3::Tag::$par; 1"
+  }
+  return;
+}
+load_parents() unless $ENV{MP3TAG_SKIP_LOCAL};
+
 1;
 
 =pod
@@ -1969,6 +2260,52 @@ byte-encoded UTF-8 (every Perl character contains a byte of UTF-8
 encoded string); otherwise (default) it is a native Perl Unicode
 string.
 
+If C<MP3TAG_SKIP_LOCAL> is true, local customization files are not loaded.
+
+=head1 CUSTOMIZATION
+
+Many aspects of operation of this module are subject to certain subtle
+choices.  A lot of effort went into making these choices customizable,
+by setting global or per-object configuration variables.
+
+A certain degree of customization of global configuration variables is
+available via the environment variables.  To make customization as
+flexible as possible, I<ALL> aspects of operation of C<MP3::Tag> are
+subject to local override.  Three customization modules
+
+  MP3::Tag::User	MP3::Tag::Site		MP3::Tag::Vendor
+
+are attempted to be loaded if present.  Only the first module (of
+those present) is loaded directly; to ensure that the whole hierarchy
+is loaded, the first thing a customization module should do is to call
+
+  MP3::Tag->load_parents()
+
+method.
+
+The customization modules have an opportunity to change global
+configuration variables on load.  To allow more flexibility, they may
+override any method defined in C<MP3::Tag>; as usual, the overriden
+method may be called using C<SUPER> modifier (see L<perlobj/"Method
+invocation">).
+
+E.g., it is recommended to make a local customization file with
+
+  eval 'require Music_Translate_Fields';
+  for my $elt ( qw( title track artist album comment year genre
+		    title_track artist_collection person ) ) {
+    no strict 'refs';
+    MP3::Tag->config("translate_$elt", \&{"Music_Translate_Fields::translate_$elt"})
+      if defined &{"Music_Translate_Fields::translate_$elt"};
+  }
+  MP3::Tag->config("short_person", \&Music_Translate_Fields::short_person)
+      if defined &Music_Translate_Fields::short_person;
+
+and install the (supplied, in the F<examples/modules>) module
+F<Music_Translate_Fields.pm> which enables normalization of person
+names (to a long or a short form), and of music piece names to
+canonical forms.
+
 =head1 EXAMPLE SCRIPTS
 
 Some example scripts come with this module:
@@ -2048,9 +2385,10 @@ C<ID3v1> tag is automatically upgraded to C<ID3v2>, the content would most
 probably be put in the "main performer", C<TPE1>, tag.  As a result, the
 content of C<TPE1> tag is also "untrusted" - it may contain, e.g., composer.
 
-In my opinion, a different field should be used for "known to be principal
-performer"; for example, the script F<mp3info2> shipped with this module
-uses C<%{TXXX[TPE1]}> in preference to C<%{TPE1}>.
+In my opinion, a different field should be used for "known to be
+principal performer"; for example, the method performer() (and the
+script F<mp3info2> shipped with this module) uses C<%{TXXX[TPE1]}> in
+preference to C<%{TPE1}>.
 
 For example, interpolate C<%{TXXX[TPE1]|TPE1}> or C<%{TXXX[TPE1]|a}> -
 this will use the frame C<TXXX> with identifier C<TPE1> if present, if not,
